<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 17, 2020
 */

/**
 * Class for installing acronis backup cloud.
 *
 * @package InterWorx
 * @subpackage CLI
 */
class AcronisDbSeeder extends CLI
{
    const DB_NAME_1 = 'testdb1';
    const DB_NAME_2 = 'testdb2';
    const DB_NAME_SYSTEM = 'testdbsystem';

    const DOMAIN = 'acronistest.com';
    const UNIQNAME = 'acronistest';

    private $_dbs = array(self::DB_NAME_2, self::DB_NAME_1);

    /**
     * Register functions, install & uninstall.
     */
    public function registerOptArgs()
    {
        $this->_func('createAllDatabases', 'create all databases - sw & system');
        $this->_func('dropAllDatabases', 'drop all databases - sw & system');
        $this->_func('createAccountAndDatabases', 'create sw account & sw databases & system db');
        $this->_func('createSw', 'create sw account');
        $this->_func('createSwDatabases', 'create & seed sw dbs');
        $this->_func('dropSwDatabases', 'drop sw dbs');
        $this->_func('createSystemDatabase', 'create system db - not connected with sw');
        $this->_func('dropSystemDatabase', 'drop system database');
    }

    /**
     * create all databases.
     */
    protected function _createAllDatabases()
    {
        $this->_createSwDbs();
        $this->_createSystemDb();

        $this->success('successfully created all dbs');
    }

    /**
     * drop all databases.
     */
    protected function _dropAllDatabases()
    {
        $this->_dropSwDbs();
        $this->_dropSystemDb();

        $this->success('successfully dropped all dbs');
    }

    /**
     * create sw account, sw databases, and system database.
     */
    protected function _createAccountAndDatabases()
    {
        $this->_createSwAccount();
        $this->_createSwDbs();
        $this->_createSystemDb();

        $this->success('successfully created all accounts & dbs');
    }

    /**
     * create sw account, db, and seed db.
     */
    protected function _createSw()
    {
        $this->_createSwAccount();
        $this->success('succsessfully created sw account');
    }

    /**
     * Create and seed dbs for sw account.
     */
    protected function _createSwDatabases()
    {
        $this->_createSwDbs();
        $this->success('successfully created databases');
    }

    /**
     * create sw dbs (creates 2 test dbs under same account).
     */
    private function _createSwDbs()
    {
        $SW = new SiteWorx(self::DOMAIN);

        foreach ($this->_dbs as $db) {
            $Reply = IW::FC()->routeFromPHP(
                'Ctrl_Siteworx_MysqlDb',
                'addCommit',
                array('name' => $db),
                $SW
            );

            if (!$Reply->wasSuccessful()) {
                IW::log($Reply->getErrors());
            } else {
                $prefixedName = self::UNIQNAME . "_$db";
                $this->_seed($prefixedName, "swtable_$db");
            }
        }
    }

    /**
     * create system database.
     */
    protected function _createSystemDatabase()
    {
        $this->_createSystemDb();
        $this->success('successfully created system database');
    }

    /**
     * create system db.
     */
    private function _createSystemDb()
    {
        $this->_createSystem();
        $this->_seed(self::DB_NAME_SYSTEM, 'systemdb');
    }

    /**
     * drop system db
     */
    private function _dropSystemDb()
    {
        $sql = "DROP DATABASE " . self::DB_NAME_SYSTEM;
        $dsn = Ini::get(Ini::MYSQL, 'rootdsn');
        $db = IWorxDbUtil::db_connect($dsn, false);

        IWorxDbUtil::db_query($db, $sql);
        $db->disconnect();
    }

    /**
     * create system db
     */
    private function _createSystem()
    {
        $sql = "CREATE DATABASE IF NOT EXISTS " . self::DB_NAME_SYSTEM;
        $dsn = Ini::get(Ini::MYSQL, 'rootdsn');
        $db = IWorxDbUtil::db_connect($dsn, false);

        IWorxDbUtil::db_query($db, $sql);
        $db->disconnect();
    }

    /**
     * Seed db.
     *
     * @param string $db
     * @param string $tableName
     */
    private function _seed($db, $tableName)
    {
        $dsn = substr(Ini::get(Ini::MYSQL, 'rootdsn'), 0, -5) . $db;
        $db = IWorxDbUtil::db_connect($dsn, false);

        $sqlCreateTable = "CREATE TABLE IF NOT EXISTS `$tableName` (`id` int(11));";
        $sqlPopulateTable = "INSERT INTO `$tableName` (`id`) VALUES (1), (2), (3);";

        IWorxDbUtil::db_query($db, $sqlCreateTable);
        IWorxDbUtil::db_query($db, $sqlPopulateTable);

        $db->disconnect();
    }

    /**
     * create sw account
     */
    private function _createSwAccount()
    {
        $this->_validateSwAccountDoesNotExist();

        $NW = new NodeWorx(NodeWorx::MASTER_ID);
        $DS = new DataSource_NW_IP_FreeIPv4($NW); //->getFirstValue();
        $ipv4 = $DS->getFirstValue();

        $acct = array(
            'master_domain' => self::DOMAIN,
            'master_domain_ipv4' => $ipv4,
            'uniqname' => self::UNIQNAME,
            'email' => 'acronistest@test.com',
            'password' => 'cronisTest123',
            'confirm_password' => 'cronisTest123',
            'OPT_STORAGE' => '999999999',
            'OPT_BANDWIDTH' => '999999999',
            'OPT_EMAIL_ALIASES' => '999999999',
            'OPT_EMAIL_AUTORESPONDERS' => '999999999',
            'OPT_EMAIL_BOXES' => '999999999',
            'OPT_EMAIL_GROUPS' => '999999999',
            'OPT_FTP_ACCOUNTS' => '999999999',
            'OPT_MYSQL_DBS' => '999999999',
            'OPT_MYSQL_DB_USERS' => '999999999',
            'OPT_POINTER_DOMAINS' => '999999999',
            'OPT_SLAVE_DOMAINS' => '999999999',
            'OPT_SUBDOMAINS' => '999999999'
        );

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'addCommit',
            $acct,
            $NW
        );

        if (!$Reply->wasSuccessful()) {
            IW::log($Reply->getErrors());
            $this->fail('unable to create sw account! ');
        }
    }

    /**
     * Fail if siteworx account already exists.
     */
    private function _validateSwAccountDoesNotExist()
    {
        $SW = new SiteWorx();
        $SW->populateUniqname(self::UNIQNAME);
        if ($SW->exists()) {
            $this->fail('Siteworx account already exists');
        }
    }

    /**
     *
     */
    protected function _dropSwDatabases()
    {
        $this->_dropSwDbs();
        $this->success('successfully dropped sw databases');
    }

    /**
     * drop databases.
     */
    private function _dropSwDbs()
    {
        $SW = new SiteWorx(self::DOMAIN);

        foreach ($this->_dbs as $db) {
            $Reply = IW::FC()->routeFromPHP(
                'Ctrl_Siteworx_MysqlDb',
                'deleteCommit',
                array('name' => $db),
                $SW
            );
        }
    }
}
