<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 07, 2020
 */

/**
 * @package InterWorx
 * @subpackage Plugin
 */
class Form_AbActivatePlan extends Form
{
    const INPUT_PLAN = 'plan_id';
    const FORM_NAME = 'ab_activate_plan';

    private $_applicablePlans;

    /**
     * Initialize.
     */
    protected function _initialize()
    {
        parent::_initialize(self::FORM_NAME);

        $this->addInput(new Input_Select(self::INPUT_PLAN))
            ->setDataSource(new DataSource_Array($this->_getPlanIds()))
            ->setHidden(true)
            ->setRequired(false);
    }

    /**
     * Get plan ids.
     *
     * @return array
     */
    private function _getPlanIds()
    {
        $plans = $this->_getApplicablePlans();

        return array_map(function ($plan) {
            return $plan['ID'];
        }, $plans);
    }

    /**
     * Get backup id.
     *
     * @return string
     */
    public function getSelectedPlanId()
    {
        return $this->getValue(self::INPUT_PLAN);
    }


    /**
     * Get default plan id. Checks if cached values exists first,
     * if not, returns best suitable plan based on pre/post scripts validation.
     *
     * @return string
     */
    public function getDefaultPlanId()
    {
        $planIds = $this->_getPlanIds();

        if (empty($planIds)) {
            throw new Exception('No plans exist');
        }

        $Cache = new Model_Cache();
        $cachedId = $Cache->getPlanId();

        if (isset($cachedId) && in_array($cachedId, $planIds, true)) {
            return $cachedId;
        } else {
            $Cache->deletePlanId();
            return $this->_getIdOfValidPlan($planIds);
        }
    }

    /**
     * Tries to get plan with valid scripts first.
     * If none exists, it returns random plan id.
     *
     * @param array $planIds
     * @return string
     */
    private function _getIdOfValidPlan(array $planIds)
    {
        foreach ($planIds as $id) {
            $Plan = new Model_Backups_BackupPlan($id);
            if ($Plan->prePostScriptsOK()) {
                return $id;
            }
        }

        throw new Exception('no valid plan was found');
    }

    /**
     * Get applicable plans.
     */
    private function _getApplicablePlans()
    {
        if (!isset($this->_applicablePlans)) {
            $Server = new Model_Server_Server();
            $this->_applicablePlans = $Server->getApplicablePlans();
        }

        return $this->_applicablePlans;
    }
}
