<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jul 12, 2021
 */

/**
 * @package InterWorx
 * @subpackage
 */
class Model_Backups_Backup
{
    const BACKUP_ID = 'backupId';
    const CREATED = 'created';
    const DATE = 'date';
    const ARCHIVE_ID = 'archiveId';
    const VAULT_ID  = 'vaultId';
    const MACHINE_ID = 'machineId';
    const LEGACY_ID = 'legacyId';
    const TENANT_AGENT_ID = 'tenantAgentId';

    const CACHE_DATE = 'date';
    const CACHE_ID = 'backup_id';
    const CACHE_API_DETAILS = 'api_details';
    const CACHE_SW_USERS = 'sw_users';
    const CACHE_TIMESTAMP = 'timestamp';
    const CACHE_MAPPED_DATABASES = 'databases_mapped';
    const CACHE_UNMAPPED_DATABASES = 'databases_unmapped';
    const CACHE_DUMPS_DIR_UNMAPPED = 'dumps_dir_unmapped';

    const NAME = 'name';
    const SIZE = 'size';
    const UNIT = 'unit';
    const UNIXUSER = 'unixuser';

    private $_cachedBackup;
    private $_created;
    private $_date;
    private $_backupId;
    private $_legacyId;
    private $_vaultId;
    private $_archiveId;
    private $_machineId;
    private $_tenantId;
    private $_swAccounts;
    private $_mappedDatabases;
    private $_unmappedDatabases;
    private $_unmappedDbFolder;

    /**
     * @param array $backupDetails
     */
    public function __construct(array $backupDetails)
    {
        $this->_created = $backupDetails[self::CREATED];
        $this->_date = $backupDetails[self::DATE];
        $this->_backupId = $backupDetails[self::BACKUP_ID];
        $this->_legacyId = $backupDetails[self::LEGACY_ID];
        $this->_tenantId = $backupDetails[self::TENANT_AGENT_ID];
        $this->_vaultId = $backupDetails[self::VAULT_ID];
        $this->_archiveId = $backupDetails[self::ARCHIVE_ID];
        $this->_machineId = $backupDetails[self::MACHINE_ID];


        if ($this->_isCached() && $this->_prescriptExists()) {
            $this->_populatePrescriptData();
        }
    }

    /**
     * Populate cached data.
     */
    private function _populatePrescriptData()
    {
        $backup = $this->_getCachedBackup();

        $this->_swAccounts = $backup[self::CACHE_SW_USERS];
        $this->_mappedDatabases = $backup[self::CACHE_MAPPED_DATABASES];
        $this->_unmappedDatabases = $backup[self::CACHE_UNMAPPED_DATABASES];
        $this->_unmappedDbFolder = $backup[self::CACHE_DUMPS_DIR_UNMAPPED];
    }

    /**
     * @return array
     */
    private function _getCachedBackup()
    {
        if (!isset($this->_cachedBackup)) {
            $Cache = new Model_Cache_Backup();
            $this->_cachedBackup = $Cache->get($this->_backupId);
        }

        return $this->_cachedBackup;
    }


    /**
     * @return boolean
     */
    private function _prescriptExists()
    {
        $Cache = new Model_Cache_Backup();
        return $Cache->prescriptCached($this->_backupId);
    }

    /**
     * Get api details.
     *
     * @return array
     */
    public function getApiDetails()
    {
        return array(
            self::DATE => $this->_date,
            self::BACKUP_ID => $this->_backupId,
            self::LEGACY_ID => $this->_legacyId,
            self::VAULT_ID => $this->_vaultId,
            self::ARCHIVE_ID => $this->_archiveId,
            self::MACHINE_ID => $this->_machineId,
            self::CREATED => $this->_created,
            self::TENANT_AGENT_ID => $this->_tenantId
        );
    }

    /**
     * Get required keys for api details.
     *
     * @return array
     */
    public static function getRequiredKeysApiDetails()
    {
        return array(
            self::DATE,
            self::BACKUP_ID,
            self::LEGACY_ID,
            self::VAULT_ID,
            self::ARCHIVE_ID,
            self::MACHINE_ID,
            self::CREATED,
            self::TENANT_AGENT_ID
        );
    }

    /**
     * @return Model_Cache_Backup
     */
    private function _getCache()
    {
        if (!isset($this->_Cache)) {
            $this->_Cache = new Model_Cache_Backup();
        }

        return $this->_Cache;
    }

    /**
     * @return string
     */
    public function getStartTime()
    {
        $time = strtotime($this->_created);
        return Model_Acronis::getDateString($time);
    }

    /**
     * @return integer
     */
    public function getTimeStamp()
    {
        return strtotime($this->_created);
    }

    /**
     * @return string
     */
    public function getBackupId()
    {
        return $this->_backupId;
    }

    /**
     * @return string
     */
    public function getLegacyId()
    {
        return $this->_legacyId;
    }

    /**
     * @return string
     */
    public function getVaultId()
    {
        return $this->_vaultId;
    }

    /**
     * @return string
     */
    public function getArchiveId()
    {
        return $this->_archiveId;
    }

    /**
     * @return string
     */
    public function getMachineId()
    {
        return $this->_machineId;
    }

    /**
     * @return string
     */
    public function getTenantAgentId()
    {
        return $this->_tenantId;
    }

    /**
     * @return string
     */
    public function getCreated()
    {
        return $this->_created;
    }

    /**
     * @return array
     */
    public function getSwAccounts()
    {
        $this->_validateCache();
        return $this->_swAccounts;
    }

    /**
     * @param string $unixuser
     * @return array
     */
    public function getUserDatabases($unixuser)
    {
        $this->_validateCache();
        $this->_validateUser($unixuser);

        $userDbs = array();
        foreach ($this->_mappedDatabases as $db) {
            if ($db[self::UNIXUSER] === $unixuser) {
                $userDbs[] = $db;
            }
        }

        return $userDbs;
    }

    /**
     * @param string $user
     * @throws Exception
     */
    private function _validateUser($user)
    {
        if (!in_array($user, $this->_swAccounts)) {
            throw new Exception("Acronis: unixuser $user did not exist at the time of the backup " . $this->getCreated());
        }
    }

    /**
     * @return array
     */
    public function getMappedDatabases()
    {
        $this->_validateCache();
        return $this->_mappedDatabases;
    }

    /**
     * @return array
     */
    public function getUnmappedDatabases()
    {
        $this->_validateCache();
        return $this->_unmappedDatabases;
    }

    /**
     * @throws Exception
     */
    private function _validateCache()
    {
        if (!$this->_prescriptExists()) {
            $message = 'Acronis:  Prescript did not capture additional information for backup ' . $this->getCreated() . '. Backup ID: ' . $this->getBackupId();
            throw new Exception($message);
        }
    }

    /**
     * Add backup to cache.
     */
    public function addToCache()
    {
        if (!$this->_isCached() && !$this->_expired()) {
            $Cache = $this->_getCache();
            $Cache->add($this->_backupId, $this->_getCacheFormat());
        }
    }

    /**
     * @return boolean
     */
    private function _expired()
    {
        $Cache = $this->_getCache();
        $timestamp = $this->getTimeStamp();

        return $Cache->expired($timestamp);
    }

    /**
     * @return boolean
     */
    private function _isCached()
    {
        $Cache = $this->_getCache();
        $id  = $this->getBackupId();

        return $Cache->exists($id);
    }

    /**
     * Get backup in cache format.
     */
    private function _getCacheFormat()
    {
        return array(
            Model_Cache_Abstract::BACKUP_ID => $this->getBackupId(),
            Model_Cache_Abstract::API_DETAILS => $this->getApiDetails()
        );
    }

    /**
     * @param array $prescriptData
     */
    public function addPrescriptDataToCache(array $prescriptData)
    {
        if (!$this->_isCached()) {
            throw new Exception_Acronis_InvalidCache('backup not cached!');
        }

        $Cache = $this->_getCache();
        $Cache->addPrescriptData($this->_backupId, $prescriptData);
    }


    /**
     * @param string $user
     */
    public function swAccountExists($user)
    {
        if ($this->_prescriptExists()) {
            $swUsers = $this->getSwAccounts();
            return in_array($user, $swUsers);
        }

        return false;
    }

    /**
     * Get unmapped db folder name.
     *
     * @return string
     */
    public function getUnmappedDbFolder()
    {
        if (!isset($this->_unmappedDbFolder)) {
            // defaults to system for old backups
            // that did not have configurable folder
            // name in prescript
            return 'system';
        }

        return $this->_unmappedDbFolder;
    }

    /**
     * @param array  $dbs
     * @param string $unixuser
     */
    public function getSizeDatabases(array $dbs, $unixuser)
    {
        $this->_validateDatabases($dbs, $unixuser);

        $dbsCache = $this->getUserDatabases($unixuser);
        $items = array();
        $totalBytes = 0;

        foreach ($dbsCache as $db) {
            $this->_validateSizeCached($db);

            $dbName = $db[self::NAME];
            $dbSize = $db[self::SIZE];

            if (in_array($dbName, $dbs)) {
                $totalBytes += $dbSize;
                $items[] = AcronisReply_Restore::getRestoreItemDb($dbName, $dbSize);
            }
        };

        return AcronisReply_Restore::getRestoreItems($items, $totalBytes);
    }

    /**
     * @param array  $dbs
     * @param string $unixuser
     */
    private function _validateDatabases(array $dbs, $unixuser)
    {
        foreach ($dbs as $db) {
            if (!$this->_userDbExists($db, $unixuser)) {
                throw new Exception_Acronis_DbNotFound($db);
            }
        }
    }

    /**
     * @param string $dbName
     * @param string $unixuser
     */
    private function _userDbExists($dbName, $unixuser)
    {
        $allDbs = $this->getMappedDatabases();

        foreach ($allDbs as $db) {
            $dbExists = $db[self::NAME] === $dbName;
            $isUserDb = $db[self::UNIXUSER] === $unixuser;

            if ($dbExists && $isUserDb) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param array $db
     */
    private function _validateSizeCached(array $db)
    {
        if (!is_numeric($db[self::SIZE])) {
            throw new Exception_Acronis_Backup_NoDbSize();
        }
    }
}
