<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 21, 2020
 */

/**
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Recovery_Files_Databases_Siteworx extends Model_Recovery_Files_Databases
{
    private $_unixUser;
    private $_SW;
    private $_id;

    /**
     * @param string $backupId
     * @param string $unixUser
     * @param string $restoreId
     */
    public function __construct($backupId, $unixUser, $restoreId = null)
    {
        $SW = new SiteWorx();
        $SW->populateUniqname($unixUser);
        if (!$SW->exists()) {
            $this->_setFailed('SiteWorx account does not exist for unix user: ' . $unixUser);
            throw new Exception_Acronis_AccountDoesNotExist($unixUser);
        }

        $this->_SW = $SW;
        $this->_unixUser = $unixUser;

        parent::__construct($backupId, $restoreId);
    }

    /**
     * Get databases associated with sw user.
     *
     * @return array
     */
    public function getDbs()
    {
        $Cache = $this->_getBackupCache();
        $id = $this->getBackupId();

        if ($Cache->prescriptCached($id)) {
            return $this->_getDbsFromPrescript();
        }

        // uses old cache system - retrieves db names from
        // acronis api by parsing folder names
        return $this->_getDbsFromApi();
    }

    /**
     * @return array
     */
    private function _getDbsFromPrescript()
    {
        $backupId = $this->getBackupId();
        $Cache = $this->_getBackupCache();
        $dbs = $Cache->getUserDatabases($backupId, $this->_unixUser);
        return $this->_getDbNames($dbs);
    }

    /**
     * Get dbs associated with sw user from acronis api
     * by parsing folder structure (db names are same as
     * folder names).
     *
     * @return array
     */
    private function _getDbsFromApi()
    {
        $dbs = $this->_getDbsFromCache();

        if (empty($dbs)) {
            $this->_cacheDumpsFromApi();
            $dbs = $this->_getDbsFromCache();
        }

        return $this->_getUserDbs($dbs);
    }

    /**
     * get dbs from "old" cache (if no data was cashed in prescript).
     *
     * @return array
     */
    private function _getDbsFromCache()
    {
        $Cache = $this->_getCache();
        return $Cache->getDumpsInfo($this->getBackupId());
    }

    /**
     * Get user dbs from array of users.
     *
     * @param array $dbsAllUsers
     * @return array
     */
    private function _getUserDbs(array $dbsAllUsers)
    {
        foreach ($dbsAllUsers as $dbsUser) {
            if ($dbsUser[self::DUMP_INFO_USER] === $this->_unixUser) {
                return $dbsUser[self::DUMP_INFO_DBS];
            }
        }

        $this->_setFailed('Unix user does not exist: ' . $this->_unixUser);
        throw new Exception_Acronis_AccountDoesNotExist($this->_unixUser);
    }

    /**
     * Restore databases.
     *
     * WARNING - this might take up to an hour to execute.
     * Run bin script to run in background.
     *
     * @param array $databases
     */
    public function restoreDatabases(array $databases)
    {
        $this->_addToRestoreCache($databases, $this->_unixUser);

        if (!$this->_userDumpsRestored($this->_unixUser)) {
            $this->_restoreUserDumps($this->_unixUser);
            $this->_awaitUserDumpsRestoration($this->_unixUser);
        }

        $this->_handleRestore($databases);
    }

    /**
     * Restore databases for user.
     *
     * @param array $databases
     */
    private function _handleRestore(array $databases)
    {
        IWorxLog::log('Acronis: Database restores in progress for user ' . $this->_unixUser);
        $this->_setInProgress();

        $this->_createSwDumpsDir();
        $this->_placeDumpsInSwUserFolder($databases);
        $this->_restoreDbs($databases);
        $this->_cleanUpSqlFiles();

        $this->_setComplete();
    }

    /**
     * Create dumps dir.
     */
    private function _createSwDumpsDir()
    {
        $swDumpsPath = $this->_getSwDumpsPath();

        if (!is_dir($swDumpsPath)) {
            mkdir($swDumpsPath, 0755);
        }
    }

    /**
     * Place dumps in sw location.
     *
     * @param array $databases
     */
    private function _placeDumpsInSwUserFolder(array $databases)
    {
        $dumpsPath = $this->_getRestoredDumpsPath($this->_unixUser);

        foreach ($databases as $dbName) {
            $archives = $this->_getTableArchives("$dumpsPath/$dbName");
            $this->_unzipFilesInSwLocation($dbName, $archives);
        }
    }

    /**
     * unzip compressed sql files and place in sw location.
     *
     * @param string $dbName
     * @param array  $archives
     */
    private function _unzipFilesInSwLocation($dbName, array $archives)
    {
        foreach ($archives as $archive) {
            $destination = $this->_getSwDestinationPath($dbName, $archive);
            $this->_uncompress($archive, $destination);
        }
    }

    /**
     * Get file name from gz path.
     *
     * @param string $dbName
     * @param string $archive
     * @return string
     */
    private function _getSwDestinationPath($dbName, $archive)
    {
        $uncompressedName = $this->_getUncompressedName($archive);

        $swPath = $this->_getSwDumpsPath() . "/$dbName";
        if (!is_dir($swPath)) {
            mkdir($swPath, 0755);
        }

        return "$swPath/$uncompressedName";
    }

    /**
     * get uncompressed filename.
     *
     * @param string $archive
     */
    private function _getUncompressedName($archive)
    {
        $ext = pathinfo($archive, PATHINFO_EXTENSION);

        if ($ext == 'gz') {
            $extensionExcluded = rtrim($archive, '.gz');
        }
        if ($ext == 'xz') {
            $extensionExcluded = rtrim($archive, '.xz');
        }

        $startPos = strrpos($archive, '/') + 1;
        $uncompressedName = substr($extensionExcluded, $startPos);
        return $uncompressedName;
    }

    /**
     * Get sw dumps path.
     *
     * @return string
     */
    private function _getSwDumpsPath()
    {
        return "/home/$this->_unixUser/var/db-dumps";
    }


    /**
     * Restore sw dbs.
     *
     * @param array $databases
     */
    private function _restoreDbs(array $databases)
    {
        $sqlPath = $this->_getSwDumpsPath();
        foreach ($databases as $dbName) {
            $dbDumpsPath = "$sqlPath/$dbName";
            $sqlFiles = $this->_getSqlFiles($dbDumpsPath, $dbName);
            $this->_restoreDatabase($dbName, $sqlFiles);
        }
    }

    /**
     * Restore database.
     *
     * @param string $database
     * @param array  $sqlFiles
     */
    private function _restoreDatabase($database, array $sqlFiles)
    {
        if (empty($sqlFiles)) {
            IWorxLog::log('Acronis: Sql files could not be restored for database ' . $database . '. The database did not exist at the time of the backup.', IWorxLog::ERR);
            $this->_setFailed('Sql dump files do not exist for database: ' . $database);
            throw new Exception('Acronis: Cannot restore databases - sql files missing');
        }
        $Driver = $this->_SW->getMysqlDriver();
        $success = $Driver->importSqlFiles($sqlFiles, $database, $this->_SW);

        if (!$success) {
            $this->_setFailed('Failed to restore database: ' . $database);
            throw new Exception('Acronis: Unable to restore database: ' . $database);
        }
    }

    /**
     * Delete sql files in sw directory.
     */
    private function _cleanUpSqlFiles()
    {
        $path = $this->_getSwDumpsPath();
        $Command = new Command("rm -rf $path/*");
        $Command->allowAsterisk();
        $Command->exec();

        $this->_cleanupDumps($this->_unixUser);
    }

    /**
     * Resore databases to sw alt location.
     *
     * @param array $dbNames
     */
    public function restoreDatabasesAltLocation(array $dbNames)
    {
        $newDirName = date("Y-m-d-H-i-s");
        $dest = Model_Recovery_Files_Siteworx::getDefaultRestoreLocation($this->_SW) . "/$newDirName";

        $this->_restoreUserDbsToAltLocation($this->_unixUser, $dbNames, $dest);
    }
}
