<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 20, 2020
 */

/**
 * Resource class.
 *
 * @package Interworx
 * @subpackage Plugin
 */
class Model_Server_Server extends Model_Acronis
{
    const NEXTBACKUP = 'nextBackup';
    const LASTBACKUP = 'lastBackup';
    const LASTSTATUS = 'lastStatus';
    const RUNNING = 'running';

    private $_resourceId;
    private $_instanceId;

    private $_lastBackup;
    private $_nextBackup;
    private $_online;
    private $_status;


    /**
     * Get plan id.
     *
     * @return string
     * @throws Exception // if no plan is assigned.
     */
    public function getPlanId()
    {
        $planId = $this->_getAssignedPlanId();
        return $planId;
    }

    /**
     * Get all backup plans.
     */
    public function getAllPlans()
    {
        $Api = $this->_getApi();
        return $Api->getBackupPlans($this->getInstanceId());
    }

    /**
     * Get id from any of the already assigned plans.
     *
     * A resource can have multiple backup plans.
     *
     * Will return plan if cached value exists in api.
     *
     * If not, looks for plan with valid pre/post script,
     * or any if no plans with valid scripts exists.
     *
     * @return string
     * @throws Exception
     */
    private function _getAssignedPlanId()
    {
        $Api = $this->_getApi();
        $plans = $Api->getBackupPlans($this->getInstanceId());

        if (empty($plans)) {
            throw new Exception_Acronis_BackupPlan_NoneAssigned('no plans are assigned to the resource.');
        }

        try {
            // first, get cached if exists
            // if none, return any plan with
            // valid pre/post scripts
            $plan = $this->_getCachedOrValid($plans);
        } catch (Exception $e) {
            // return id from random plan
            // assigned to the resource.
            $plan = $plans[0];
        } finally {
            $planId = $plan['planId'];
            $this->_getCache()->cachePlanId($planId);
            return $planId;
        }
    }

    /**
     * First try to retrieve plan with cached id.
     *
     * @param array $plans
     */
    private function _getCachedOrValid(array $plans)
    {
        $Cache = $this->_getCache();
        $cachedPlanId = $Cache->getPlanId();

        try {
            $plan = $this->_getPlanWithId($cachedPlanId, $plans);
            return $plan;
        } catch (Exception $e) {
            $Cache->deletePlanId();
            $plan = $this->_getPlanWithValidScripts($plans);
            return $plan;
        }
    }


    /**
     * return plan if scripts are ok.
     *
     * @param array $plans
     * @return array
     */
    private function _getPlanWithValidScripts(array $plans)
    {
        foreach ($plans as $plan) {
            // acronis api will refer to plan id
            // differently depending on api endpoint
            $planId = isset($plan['planId']) ? $plan['planId'] : $plan['ID'];

            // validate plan if none cached.
            $planOk = $this->_prePostScriptsOk($planId);

            if ($planOk) {
                return $plan;
            }
        }

        throw new Exception('no valid plan found!');
    }


    /**
     * Get id of applicable plan / plan that is not assigned to resource already.
     * Will not return plan unless 1) cached, or 2) has valid pre/post scripts
     *
     * @return string
     * @throws Exception // if no applicable plans are found.
     */
    public function getApplicablePlanId()
    {
        $Api = $this->_getApi();
        $applicablePlans = $Api->getApplicablePlans($this->getResourceId())['Plans'];

        try {
            $id = $this->_getCachedPlanId($applicablePlans);
        } catch (Exception $e) {
            $this->_getCache()->deletePlanId();
            $id = $this->_getValidPlanId($applicablePlans);
        } finally {
            // will not be set if exception
            // thrown in catch block.
            if (isset($id)) {
                return $id;
            }
        }
    }

    /**
     * get plan id that matches cached value.
     * Throws error if none is saved in cache,
     * or if matching plan can't be found in api.
     *
     * @param array $plans
     */
    private function _getCachedPlanId(array $plans)
    {
        $cachedId = $this->_getCache()->getPlanId();

        if (!isset($cachedId)) {
            throw new Exception('Plan id not cached');
        }

        $plan = $this->_getPlanWithId($cachedId, $plans);
        $id = isset($plan['planId']) ? $plan['planId'] : $plan['ID'];

        return $id;
    }

    /**
     * @param string $id
     * @param array  $plans
     */
    private function _getPlanWithId($id, array $plans)
    {
        if ($id === null) {
            throw new Exception('Id not provided');
        }

        foreach ($plans as $plan) {
            $planId = isset($plan['planId']) ? $plan['planId'] : $plan['ID'];

            if ($planId === $id) {
                return $plan;
            }
        }

        throw new Exception('no plan exists with id: ' . $id);
    }

    /**
     * Get valid plan id.
     *
     * @param array $plans
     * @throws Exception
     */
    private function _getValidPlanId(array $plans)
    {
        $plan = $this->_getPlanWithValidScripts($plans);
        $id = isset($plan['planId']) ? $plan['planId'] : $plan['ID'];
        return $id;
    }

    /**
     * Get applicable plans.
     */
    public function getApplicablePlans()
    {
        $Api = $this->_getApi();
        return $Api->getApplicablePlans($this->getResourceId())['Plans'];
    }

    /**
     * Make sure pre/post scripts are enabled and configured.
     *
     * @param string $planId
     */
    protected function _prePostScriptsOk($planId)
    {
        $BackupPlan = $this->_getPlugin()->getBackupPlan($planId);
        return $BackupPlan->prePostScriptsOK();
    }

    /**
     * Get resource backup status. includes last backup, next backup, last backup status.
     *
     * @return array
     */
    public function getBackupStatus()
    {
        $Api = $this->_getApi();
        $instanceId = $this->getInstanceId();

        $apiRes = $Api->getBackupPlans($instanceId);
        $status = $this->_formatBackupStatus($apiRes[0]);

        return $status;
    }

    /**
     * Get next backup.
     *
     * @return boolean
     */
    public function backupNotScheduled()
    {
        if ($this->isProtected()) {
            $backupStatus = $this->getBackupStatus();
            $nextBackup = $backupStatus[self::NEXTBACKUP];
            return empty($nextBackup);
        }

        return true;
    }

    /**
     * check if backup in progress.
     */
    public function backupInProgress()
    {
        $backupStatus = $this->getBackupStatus();
        $lastStatus = $backupStatus[self::LASTSTATUS];
        return $lastStatus === self::RUNNING;
    }

    /**
     * Format backup status object.
     *
     * @param array $backupStatus
     * @return array
     */
    private function _formatBackupStatus(array $backupStatus)
    {
        return array(
            self::LASTSTATUS => $backupStatus['lastStatus'],
            self::LASTBACKUP => self::formatDate($backupStatus['lastStartTime']),
            self::NEXTBACKUP => self::formatDate($backupStatus['nextStartTime'])
        );
    }


    /**
     * Get resource id.
     *
     * @return string
     */
    public function getResourceId()
    {
        if (!isset($this->_resourceId)) {
            $this->_setResourceIdFromCacheOrApi();
        }

        return $this->_resourceId;
    }

    /**
     * set resource id either from cached value or by api request.
     */
    private function _setResourceIdFromCacheOrApi()
    {
        $this->_resourceId = $this->_getCache()->getResourceId();

        if (!isset($this->_resourceId)) {
            $this->_setDeviceInfo();
        }
    }

    /**
     * Get instance id.
     *
     * @return string
     */
    public function getInstanceId()
    {
        if (!isset($this->_instanceId)) {
            $this->_setInstanceIdFromCacheOrApi();
        }

        return $this->_instanceId;
    }

    /**
     * Set instance id either from cache value or by api req.
     */
    private function _setInstanceIdFromCacheOrApi()
    {
        $this->_instanceId = $this->_getCache()->getInstanceId();

        if (!isset($this->_instanceId)) {
            $this->_setDeviceInfo();
        }
    }

    /**
     * Get last date string.
     *
     * @return string
     */
    public function getLastBackup()
    {
        if (!isset($this->_lastBackup)) {
            $this->_setDeviceInfo();
        }

        return self::formatDate($this->_lastBackup);
    }

    /**
     * Get next backup date string.
     *
     * @return string
     */
    public function getNextBackup()
    {
        if (!isset($this->_nextBackup)) {
            $this->_setDeviceInfo();
        }

        return self::formatDate($this->_nextBackup);
    }


    /**
     * Check if machine is online.
     *
     * @return boolean
     */
    public function isOnline()
    {
        if (!isset($this->_online)) {
            $this->_setDeviceInfo();
        }

        return $this->_online;
    }

    /**
     * Get last backup status.
     *
     * @return string
     */
    public function isProtected()
    {
        $this->_setDeviceInfo();
        return $this->_status !== 'not_protected';
    }

    /**
     * Get device storage size.
     *
     * @return array
     */
    public function getStorageData()
    {
        if (!isset($this->_instanceId)) {
            $this->_setDeviceInfo();
        }

        $Api = $this->_getApi();
        $storageInfo = $Api->getStorageData($this->_instanceId);
        $gb = $this->_convertBytes($storageInfo['occupied_data_size']);
        return array('size' => $gb);
    }


    /**
     * Set device info.
     */
    private function _setDeviceInfo()
    {
        $device = $this->_getCache()->getDeviceInfo();

        if (empty($device)) {
            $Api = $this->_getServerApi();
            $apiResponse = $Api->getDevice();
            $device = $this->_formatDeviceInfoFromApi($apiResponse);
            $this->_getCache()->cacheDeviceInfo($device);
        }

        $this->_resourceId = $device['resourceId'];
        $this->_lastBackup = $device['lastBackup'];
        $this->_nextBackup = $device['nextBackup'];
        $this->_online = $device['online'];
        $this->_instanceId = $device['instanceId'];
        $this->_status = $device['status'];

        $this->_getCache()->cacheResourceId($this->_resourceId);
        $this->_getCache()->cacheInstanceId($this->_instanceId);
    }

    /**
     * @return Model_Server_Api
     */
    private function _getServerApi()
    {
        return $this->_getApi();
    }

    /**
     *
     * Format device info from api.
     *
     * @param array $deviceInfo
     * @return array
     */
    private function _formatDeviceInfoFromApi(array $deviceInfo)
    {
        return array(
            'resourceId' => $deviceInfo['id'],
            'lastBackup' => $deviceInfo['lastBackup'],
            'nextBackup' => $deviceInfo['nextBackup'],
            'online' => $deviceInfo['online'],
            'instanceId' => $deviceInfo['instanceId'],
            'status' => $deviceInfo['status']
        );
    }
}
